# Copyright (C) 2016 A10 Networks Inc. All rights reserved.

EXTENSION = 'a10-scaling-group'

SERVICE = "A10_SCALING_GROUP"

SCALING_GROUPS = 'a10_scaling_groups'
SCALING_GROUP = 'a10_scaling_group'

SCALING_GROUP_WORKERS = 'a10_scaling_group_workers'
SCALING_GROUP_WORKER = 'a10_scaling_group_worker'

SCALING_POLICIES = 'a10_scaling_policies'
SCALING_POLICY = 'a10_scaling_policy'

SCALING_ALARMS = 'a10_scaling_alarms'
SCALING_ALARM = 'a10_scaling_alarm'

SCALING_ACTIONS = 'a10_scaling_actions'
SCALING_ACTION = 'a10_scaling_action'

ALARM_UNITS = ['count', 'percentage', 'bytes']
ALARM_AGGREGATIONS = ['avg', 'min', 'max', 'sum']
ALARM_MEASUREMENTS = ['connections', 'memory', 'cpu', 'interface']
ALARM_OPERATORS = ['>=', '>', '<=', '<']
ALARM_PERIOD_UNITS = ['minute', 'hour', 'day']
ACTIONS = ['scale-in', 'scale-out']

RESOURCE_ATTRIBUTE_MAP = {
    SCALING_GROUPS: {
        'id': {
            'allow_post': False,
            'allow_put': True,
            'validate': {
                'type:uuid': None
            },
            'is_visible': True,
            'primary_key': True
        },
        'tenant_id': {
            'allow_post': True,
            'allow_put': False,
            'required_by_policy': True,
            'is_visible': True
        },
        'name': {
            'allow_post': True,
            'allow_put': True,
            'validate': {
                'type:string': None
            },
            'is_visible': True,
            'default': ''
        },
        'description': {
            'allow_post': True,
            'allow_put': True,
            'validate': {
                'type:string': None
            },
            'is_visible': True,
            'default': '',
        },
        'scaling_policy_id': {
            'allow_post': True,
            'allow_put': True,
            'validate': {
                'a10_type:nullable': {
                    'type:uuid': None,
                    'a10_type:reference': SCALING_POLICY
                }
            },
            'is_visible': True,
            'default': lambda attr: attr.ATTR_NOT_SPECIFIED
        }
    },
    SCALING_GROUP_WORKERS: {
        'id': {
            'allow_post': False,
            'allow_put': True,
            'validate': {
                'type:uuid': None
            },
            'is_visible': True,
            'primary_key': True
        },
        'tenant_id': {
            'allow_post': True,
            'allow_put': False,
            'required_by_policy': True,
            'is_visible': True
        },
        'name': {
            'allow_post': True,
            'allow_put': True,
            'validate': {
                'type:string': None
            },
            'is_visible': True,
            'default': ''
        },
        'description': {
            'allow_post': True,
            'allow_put': True,
            'validate': {
                'type:string': None
            },
            'is_visible': True,
            'default': '',
        },
        'scaling_group_id': {
            'allow_post': True,
            'allow_put': False,
            'validate': {
                'type:uuid': None,
                'a10_type:reference': SCALING_GROUP
            },
            'is_visible': True
        },
        'host': {
            'allow_post': False,
            'allow_put': True,
            'validate': {
                'type:string': None
            },
            'is_visible': True
        },
        'username': {
            'allow_post': False,
            'allow_put': True,
            'validate': {
                'type:string': None
            },
            'is_visible': True
        },
        'password': {
            'allow_post': False,
            'allow_put': True,
            'validate': {
                'type:string': None
            },
            'is_visible': False
        },
        'api_version': {
            'allow_post': False,
            'allow_put': True,
            'validate': {
                'type:values': ['2.1', '3.0']
            },
            'is_visible': True
        },
        'protocol': {
            'allow_post': False,
            'allow_put': True,
            'validate': {
                'type:values': ['http', 'https']
            },
            'convert_to': lambda attr: convert_to_lower,
            'is_visible': True,
            'default': lambda attr: attr.ATTR_NOT_SPECIFIED
        },
        'port': {
            'allow_post': False,
            'allow_put': True,
            'validate': {
                'type:range': [0, 65535]
            },
            'convert_to': lambda attr: attr.convert_to_int,
            'is_visible': True,
            'default': lambda attr: attr.ATTR_NOT_SPECIFIED
        },
        'nova_instance_id': {
            'allow_post': False,
            'allow_put': False,
            'validate': {
                'type:uuid': None
            },
            'is_visible': True,
            'default': lambda attr: attr.ATTR_NOT_SPECIFIED
        }
    },
    SCALING_POLICIES: {
        'id': {
            'allow_post': False,
            'allow_put': True,
            'validate': {
                'type:uuid': None
            },
            'is_visible': True,
            'primary_key': True
        },
        'tenant_id': {
            'allow_post': True,
            'allow_put': False,
            'required_by_policy': True,
            'is_visible': True
        },
        'name': {
            'allow_post': True,
            'allow_put': True,
            'validate': {
                'type:string': None
            },
            'is_visible': True,
            'default': ''
        },
        'description': {
            'allow_post': True,
            'allow_put': True,
            'validate': {
                'type:string': None
            },
            'is_visible': True,
            'default': '',
        },
        'cooldown': {
            'allow_post': True,
            'allow_put': True,
            'validate': {
                'type:non_negative': None
            },
            'convert_to': lambda attr: attr.convert_to_int,
            'is_visible': True,
            'default': 300,
        },
        'min_instances': {
            'allow_post': True,
            'allow_put': True,
            'validate': {
                'type:non_negative': None
            },
            'convert_to': lambda attr: attr.convert_to_int,
            'is_visible': True,
            'default': 1,
        },
        'max_instances': {
            'allow_post': True,
            'allow_put': True,
            'validate': {
                'a10_type:nullable': {
                    'type:non_negative': None
                }
            },
            'convert_to': lambda attr: convert_nullable(attr.convert_to_int),
            'is_visible': True,
            'default': lambda attr: attr.ATTR_NOT_SPECIFIED
        },
        'reactions': {
            'allow_post': True,
            'allow_put': True,
            'convert_list_to': lambda attr: attr.convert_kvp_list_to_dict,
            'is_visible': True,
            'default': lambda attr: attr.ATTR_NOT_SPECIFIED
        }
    },
    SCALING_ALARMS: {
        'id': {
            'allow_post': False,
            'allow_put': True,
            'validate': {
                'type:uuid': None
            },
            'is_visible': True,
            'primary_key': True
        },
        'tenant_id': {
            'allow_post': True,
            'allow_put': False,
            'required_by_policy': True,
            'is_visible': True
        },
        'name': {
            'allow_post': True,
            'allow_put': True,
            'validate': {
                'type:string': None
            },
            'is_visible': True,
            'default': ''
        },
        'description': {
            'allow_post': True,
            'allow_put': True,
            'validate': {
                'type:string': None
            },
            'is_visible': True,
            'default': '',
        },
        'aggregation': {
            'allow_post': True,
            'allow_put': True,
            'validate': {
                'type:values': ['avg', 'min', 'max', 'sum']
            },
            'is_visible': True,
            'convert_to': lambda attr: convert_to_lower,
            'default': 'avg'
        },
        'measurement': {
            'allow_post': True,
            'allow_put': True,
            'validate': {
                'type:values': ['connections', 'memory', 'cpu', 'interface']
            },
            'convert_to': lambda attr: convert_to_lower,
            'is_visible': True
        },
        'operator': {
            'allow_post': True,
            'allow_put': True,
            'validate': {
                'type:values': ['>=', '>', '<=', '<']
            },
            'is_visible': True
        },
        'threshold': {
            'allow_post': True,
            'allow_put': True,
            'validate': {
                'a10_type:float': None
            },
            'convert_to': lambda attr: convert_to_float,
            'is_visible': True
        },
        'unit': {
            'allow_post': True,
            'allow_put': True,
            'validate': {
                'type:values': ['count', 'percentage', 'bytes']
            },
            'convert_to': lambda attr: convert_to_lower,
            'is_visible': True
        },
        'period': {
            'allow_post': True,
            'allow_put': True,
            'validate': {
                'type:non_negative': None
            },
            'convert_to': lambda attr: attr.convert_to_int,
            'is_visible': True,
        },
        'period_unit': {
            'allow_post': True,
            'allow_put': True,
            'validate': {
                'type:values': ['minute', 'hour', 'day']
            },
            'convert_to': lambda attr: convert_to_lower,
            'is_visible': True
        }
    },
    SCALING_ACTIONS: {
        'id': {
            'allow_post': False,
            'allow_put': True,
            'validate': {
                'type:uuid': None
            },
            'is_visible': True,
            'primary_key': True
        },
        'tenant_id': {
            'allow_post': True,
            'allow_put': False,
            'required_by_policy': True,
            'is_visible': True
        },
        'name': {
            'allow_post': True,
            'allow_put': True,
            'validate': {
                'type:string': None
            },
            'is_visible': True,
            'default': ''
        },
        'description': {
            'allow_post': True,
            'allow_put': True,
            'validate': {
                'type:string': None
            },
            'is_visible': True,
            'default': '',
        },
        'action': {
            'allow_post': True,
            'allow_put': True,
            'validate': {
                'type:values': ['scale-in', 'scale-out']
            },
            'convert_to': lambda attr: convert_to_lower,
            'is_visible': True
        },
        'amount': {
            'allow_post': True,
            'allow_put': True,
            'validate': {
                'type:non_negative': None
            },
            'convert_to': lambda attr: attr.convert_to_int,
            'is_visible': True,
        },
    }
}


def convert_to_lower(input):
    try:
        return input.lower()
    except AttributeError:
        return input


def convert_to_float(input):
    try:
        return float(input)
    except ValueError:
        return input


def convert_nullable(convert_value):
    def f(input):
        if input is not None:
            return convert_value(input)
        return None
    return f


def validate_float(data, options):
    if not isinstance(data, float):
        return "'%s' is not a number" % input


def validate_reference(data, options):
    """Referential integrity is enforced by the data model"""
    return None


def validate_nullable(validators):
    def f(data, options):
        if data is not None:
            for rule in options:
                value_validator = validators[rule]
                reason = value_validator(data, options[rule])
                if reason:
                    return reason
    return f


VALIDATORS = {
    'a10_type:float': lambda validators: validate_float,
    'a10_type:reference': lambda validators: validate_reference,
    'a10_type:nullable': validate_nullable
}



# Copyright (C) 2016 A10 Networks Inc. All rights reserved.

import operator

from neutronclient.common import utils

from a10_neutronclient import client_extension
from a10_neutronclient.resources import a10_scaling_group


class ScalingGroupExtension(client_extension.ClientExtension):

    resource = a10_scaling_group.SCALING_GROUP
    resource_plural = a10_scaling_group.SCALING_GROUPS

    resource_attribute_map = a10_scaling_group.RESOURCE_ATTRIBUTE_MAP

    object_path = '/%s' % resource_plural
    resource_path = '/%s/%%s' % resource_plural
    versions = ['2.0']


class ScalingGroupList(client_extension.List, ScalingGroupExtension):
    """List A10 scaling groups"""

    shell_command = 'a10-scaling-group-list'

    list_columns = ['id', 'name', 'scaling_policy_id', 'description']


class ScalingGroupCreate(client_extension.Create, ScalingGroupExtension):
    """Create A10 scaling group"""

    shell_command = 'a10-scaling-group-create'

    list_columns = ['id', 'name', 'scaling_policy_id', 'description']

    def add_known_arguments(self, parser):
        self._add_known_arguments(parser, ['name'])


class ScalingGroupUpdate(client_extension.Update, ScalingGroupExtension):
    """Update A10 scaling group"""

    shell_command = 'a10-scaling-group-update'

    list_columns = ['id', 'name', 'scaling_policy_id', 'description']


class ScalingGroupDelete(client_extension.Delete, ScalingGroupExtension):
    """Delete A10 scaling group"""

    shell_command = 'a10-scaling-group-delete'


class ScalingGroupShow(client_extension.Show, ScalingGroupExtension):
    """Show A10 scaling group"""

    shell_command = 'a10-scaling-group-show'


class ScalingGroupWorkerExtension(client_extension.ClientExtension):

    resource = a10_scaling_group.SCALING_GROUP_WORKER
    resource_plural = a10_scaling_group.SCALING_GROUP_WORKERS

    resource_attribute_map = a10_scaling_group.RESOURCE_ATTRIBUTE_MAP

    object_path = '/%s' % resource_plural
    resource_path = '/%s/%%s' % resource_plural
    versions = ['2.0']


class ScalingGroupWorkerList(client_extension.List, ScalingGroupWorkerExtension):
    """List A10 scaling group workers"""

    shell_command = 'a10-scaling-group-worker-list'

    list_columns = ['id', 'name', 'scaling_group_id', 'nova_instance_id', 'description']


class ScalingGroupWorkerCreate(client_extension.Create, ScalingGroupWorkerExtension):
    """Create A10 scaling group worker"""

    shell_command = 'a10-scaling-group-worker-create'

    list_columns = ['id', 'name', 'scaling_group_id', 'nova_instance_id', 'description']

    def add_known_arguments(self, parser):
        self._add_known_arguments(parser, ['scaling_group_id'])


class ScalingGroupWorkerUpdate(client_extension.Update, ScalingGroupWorkerExtension):
    """Update A10 scaling group worker"""

    shell_command = 'a10-scaling-group-worker-update'

    list_columns = ['id', 'name', 'scaling_group_id', 'nova_instance_id', 'description']


class ScalingGroupWorkerDelete(client_extension.Delete, ScalingGroupWorkerExtension):
    """Delete A10 scaling group worker"""

    shell_command = 'a10-scaling-group-worker-delete'


class ScalingGroupWorkerShow(client_extension.Show, ScalingGroupWorkerExtension):
    """Show A10 scaling group worker"""

    shell_command = 'a10-scaling-group-worker-show'


class ScalingPolicyExtension(client_extension.ClientExtension):

    resource = a10_scaling_group.SCALING_POLICY
    resource_plural = a10_scaling_group.SCALING_POLICIES

    resource_attribute_map = a10_scaling_group.RESOURCE_ATTRIBUTE_MAP

    object_path = '/%s' % resource_plural
    resource_path = '/%s/%%s' % resource_plural
    versions = ['2.0']

    def add_reactions_argument(self, parser):
        parser.add_argument(
            '--reactions',
            nargs='+',
            action='append',
            dest='reactions',
            type=utils.str2dict)

    def reaction2body(self, reaction_arg):
        reaction = {}
        reaction['alarm_id'] = self.get_resource_id(
            a10_scaling_group.SCALING_ALARM,
            reaction_arg.get('alarm'))
        reaction['action_id'] = self.get_resource_id(
            a10_scaling_group.SCALING_ACTION,
            reaction_arg.get('action'))
        return reaction

    def alter_body(self, parsed_args, body):
        if 'reactions' in body:
            body['reactions'] = map(self.reaction2body, reduce(operator.add, body['reactions'], []))
        return body


class ScalingPolicyList(client_extension.List, ScalingPolicyExtension):
    """List A10 scaling policies"""

    shell_command = 'a10-scaling-policy-list'

    list_columns = ['id', 'name', 'cooldown', 'min_instances', 'max_instances', 'description']


class ScalingPolicyCreate(client_extension.Create, ScalingPolicyExtension):
    """Create A10 scaling policy"""

    shell_command = 'a10-scaling-policy-create'

    list_columns = ['id', 'name', 'cooldown', 'min_instances', 'max_instances', 'description']

    def add_known_arguments(self, parser):
        self.add_reactions_argument(parser)
        self._add_known_arguments(parser, ['name'], ignore=['reactions'])


class ScalingPolicyUpdate(client_extension.Update, ScalingPolicyExtension):
    """Update A10 scaling policy"""

    shell_command = 'a10-scaling-policy-update'

    list_columns = ['id', 'name', 'cooldown', 'min_instances', 'max_instances', 'description']

    def add_known_arguments(self, parser):
        self.add_reactions_argument(parser)
        self._add_known_arguments(parser, [], ignore=['reactions'])


class ScalingPolicyDelete(client_extension.Delete, ScalingPolicyExtension):
    """Delete A10 scaling policy"""

    shell_command = 'a10-scaling-policy-delete'


class ScalingPolicyShow(client_extension.Show, ScalingPolicyExtension):
    """Show A10 scaling policy"""

    shell_command = 'a10-scaling-policy-show'


class ScalingAlarmExtension(client_extension.ClientExtension):

    resource = a10_scaling_group.SCALING_ALARM
    resource_plural = a10_scaling_group.SCALING_ALARMS

    resource_attribute_map = a10_scaling_group.RESOURCE_ATTRIBUTE_MAP

    object_path = '/%s' % resource_plural
    resource_path = '/%s/%%s' % resource_plural
    versions = ['2.0']


class ScalingAlarmList(client_extension.List, ScalingAlarmExtension):
    """List A10 scaling alarms"""

    shell_command = 'a10-scaling-alarm-list'

    list_columns = ['id', 'name',
                    'aggregation', 'measurement',
                    'operator', 'threshold', 'unit',
                    'period', 'period_unit',
                    'description']


class ScalingAlarmCreate(client_extension.Create, ScalingAlarmExtension):
    """Create A10 scaling alarm"""

    shell_command = 'a10-scaling-alarm-create'

    list_columns = ['id', 'name',
                    'aggregation', 'measurement',
                    'operator', 'threshold', 'unit',
                    'period', 'period_unit',
                    'description']

    def add_known_arguments(self, parser):
        self._add_known_arguments(parser, ['name'])


class ScalingAlarmUpdate(client_extension.Update, ScalingAlarmExtension):
    """Update A10 scaling alarm"""

    shell_command = 'a10-scaling-alarm-update'

    list_columns = ['id', 'name',
                    'aggregation', 'measurement',
                    'operator', 'threshold', 'unit',
                    'period', 'period_unit',
                    'description']


class ScalingAlarmDelete(client_extension.Delete, ScalingAlarmExtension):
    """Delete A10 scaling alarm"""

    shell_command = 'a10-scaling-alarm-delete'


class ScalingAlarmShow(client_extension.Show, ScalingAlarmExtension):
    """Show A10 scaling alarm"""

    shell_command = 'a10-scaling-alarm-show'


class ScalingActionExtension(client_extension.ClientExtension):

    resource = a10_scaling_group.SCALING_ACTION
    resource_plural = a10_scaling_group.SCALING_ACTIONS

    resource_attribute_map = a10_scaling_group.RESOURCE_ATTRIBUTE_MAP

    object_path = '/%s' % resource_plural
    resource_path = '/%s/%%s' % resource_plural
    versions = ['2.0']


class ScalingActionList(client_extension.List, ScalingActionExtension):
    """List A10 scaling actions"""

    shell_command = 'a10-scaling-action-list'

    list_columns = ['id', 'name', 'action', 'amount', 'description']


class ScalingActionCreate(client_extension.Create, ScalingActionExtension):
    """Create A10 scaling action"""

    shell_command = 'a10-scaling-action-create'

    list_columns = ['id', 'name', 'action', 'amount', 'description']

    def add_known_arguments(self, parser):
        self._add_known_arguments(parser, ['name'])


class ScalingActionUpdate(client_extension.Update, ScalingActionExtension):
    """Update A10 scaling action"""

    shell_command = 'a10-scaling-action-update'

    list_columns = ['id', 'name', 'action', 'amount', 'description']


class ScalingActionDelete(client_extension.Delete, ScalingActionExtension):
    """Delete A10 scaling action"""

    shell_command = 'a10-scaling-action-delete'


class ScalingActionShow(client_extension.Show, ScalingActionExtension):
    """Show A10 scaling action"""

    shell_command = 'a10-scaling-action-show'



# Copyright (C) 2016 A10 Networks Inc. All rights reserved.

from neutronclient.common import extension
from neutronclient.neutron import v2_0 as neutronV20

_NEUTRON_OPTIONS = ['id', 'tenant_id']

JUST_NONE = object()


class ClientExtension(extension.NeutronClientExtension):

    def _arg_name(self, name, types, prefix="--"):
        if 'a10_type:nullable' in types:
            return self._arg_name(name, types['a10_type:nullable'], prefix)

        if 'a10_type:reference' in types:
            if name.endswith('_id'):
                name = name[:-3]

        """--shish-kabob it"""
        return prefix + name.replace('_', '-')

    def _add_known_arguments(self, parser, required, ignore=[], where=lambda x: True):
        attributes = self.resource_attribute_map[self.resource_plural]
        for name in required:
            parser.add_argument(name)
        for name, attr in attributes.items():
            if name in required or name in _NEUTRON_OPTIONS or name in ignore or not where(attr):
                continue
            types = attr.get('validate', {})
            parser.add_argument(self._arg_name(name, types), dest=name)

            if 'a10_type:nullable' in types:
                parser.add_argument(
                    self._arg_name(name, types, '--no-'),
                    action='store_const',
                    const=JUST_NONE,
                    dest=name)

    def alter_body(self, parsed_args, body):
        return body

    def _transform_arg(self, value, types):
        if value == JUST_NONE:
            return None

        if 'a10_type:nullable' in types:
            return self._transform_arg(value, types['a10_type:nullable'])

        if 'a10_type:reference' in types:
            reference_to = types['a10_type:reference']
            return self.get_resource_id(reference_to, value)

        return value

    def args2body(self, parsed_args):
        attributes = self.resource_attribute_map[self.resource_plural]
        body = {}
        neutronV20.update_dict(parsed_args, body, [a for a in attributes if a != 'id'])

        for k in body:
            types = attributes.get(k, {}).get('validate', {})
            body[k] = self._transform_arg(body[k], types)

        altered = self.alter_body(parsed_args, body)
        return {self.resource: altered}

    def get_resource_id(self, resource, name_or_id):
        client = self.get_client()
        return neutronV20.find_resourceid_by_name_or_id(
            client,
            resource,
            name_or_id)


class List(extension.ClientExtensionList):
    pagination_support = True
    sorting_support = True


class Create(extension.ClientExtensionCreate):

    def _add_known_arguments(self, parser, required, ignore=[]):
        super(Create, self)._add_known_arguments(
            parser,
            required,
            ignore=ignore,
            where=lambda attr: attr.get('allow_post'))

    def add_known_arguments(self, parser):
        return self._add_known_arguments(parser, [])


class Update(extension.ClientExtensionUpdate):

    def _add_known_arguments(self, parser, required, ignore=[]):
        super(Update, self)._add_known_arguments(
            parser,
            required,
            ignore=ignore,
            where=lambda attr: attr.get('allow_put'))

    def add_known_arguments(self, parser):
        return self._add_known_arguments(parser, [])


class Delete(extension.ClientExtensionDelete):
    pass


class Show(extension.ClientExtensionShow):
    pass



# Copyright 2015,  A10 Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

EXTENSION = 'a10-device-instance'

SERVICE = "A10_DEVICE_INSTANCE"

RESOURCES = 'a10_device_instances'
RESOURCE = 'a10_device_instance'

RESOURCE_ATTRIBUTE_MAP = {
    RESOURCES: {
        'id': {
            'allow_post': False,
            'allow_put': True,
            'validate': {
                'type:uuid': None
            },
            'is_visible': True,
            'primary_key': True
        },
        'tenant_id': {
            'allow_post': True,
            'allow_put': False,
            'required_by_policy': True,
            'is_visible': True
        },
        'name': {
            'allow_post': True,
            'allow_put': True,
            'validate': {
                'type:string': None
            },
            'is_visible': True,
            'default': ''
        },
        'description': {
            'allow_post': True,
            'allow_put': True,
            'validate': {
                'type:string': None
            },
            'is_visible': True,
            'default': '',
        },
        'host': {
            'allow_post': True,
            'allow_put': True,
            'validate': {
                'type:string': None
            },
            'is_visible': True
        },
        'username': {
            'allow_post': True,
            'allow_put': True,
            'validate': {
                'type:string': None
            },
            'is_visible': True
        },
        'password': {
            'allow_post': True,
            'allow_put': True,
            'validate': {
                'type:string': None
            },
            'is_visible': False
        },
        'api_version': {
            'allow_post': True,
            'allow_put': True,
            'validate': {
                'type:values': ['2.1', '3.0']
            },
            'is_visible': True
        },
        'protocol': {
            'allow_post': True,
            'allow_put': True,
            'validate': {
                'type:values': ['http', 'https']
            },
            'is_visible': True,
            'convert_to': lambda attr: convert_to_lower,
            'default': lambda attr: attr.ATTR_NOT_SPECIFIED
        },
        'port': {
            'allow_post': True,
            'allow_put': True,
            'validate': {
                'type:range': [0, 65535]
            },
            'convert_to': lambda attr: attr.convert_to_int,
            'is_visible': True,
            'default': lambda attr: attr.ATTR_NOT_SPECIFIED
        },
        'nova_instance_id': {
            'allow_post': True,
            'allow_put': True,
            'validate': {
                'type:uuid': None
            },
            'is_visible': True,
            'default': lambda attr: attr.ATTR_NOT_SPECIFIED
        }
    }
}


def convert_to_lower(input):
    try:
        return input.lower()
    except AttributeError:
        return input
